//https://www.shadertoy.com/view/XsVGz3
//https://www.shadertoy.com/view/XtGGRt
#include "ShaderConstants.fxh"
#define MIN_HEIGHT 3.2
#define MAX_HEIGHT 6.0


struct PS_Input
{
    float4 position : SV_Position;
    float4 color : COLOR;
	float3 cloud : CLOUD;
	float skya : SKYA;
};

struct PS_Output
{
    float4 color : SV_Target;
};

float rand(float2 p) {
    float dt = dot(p, float2(12.9898, 78.233));
    float sn = fmod(dt, 3.1415);
    return frac(sin(sn) * 400.5453);
}

float Hash(float3 p)
{
	p  = frac(p /  float3(3.07965, 7.1235, 4.998784));
    p += dot(p.xyz, p.yzx + 19.19);
    return frac(p.x * p.y * p.z);
}

float noise(float3 p)
{
    float2 add = float2(1.0, 0.0);
	float3 i = floor(p);
	float3 f = frac(p); 
	f *= f * (3.0-2.0*f);

    return lerp(
		lerp(lerp(Hash(i  			), 	Hash(i + add.xyy),f.x),
			lerp(Hash(i + add.yxy),		Hash(i + add.xxy),f.x),
			f.y),
		lerp(lerp(Hash(i + add.yyx),    	Hash(i + add.xyx),f.x),
			lerp(Hash(i + add.yxx), 		Hash(i + add.xxx),f.x),
			f.y),
		f.z);
}

float FBM(float3 p,float l)
{
    float f = 0.0;
    p += TIME*0.075;
    p = p * 3.5;
    f += 0.3 * noise(p); p = 2.0 * p;
	f += 0.25000 * noise(p); p = 2.0 * p;
	f += 0.12500 * noise(p); p = 2.0 * p;
	f += 0.06250 * noise(p); p = 2.0 * p;
    f += 0.03125 * noise(p);
    
    return smoothstep(0.0, 0.9, f);
}


float planeIntersect( float3 ro, float3 rd, float plane)
{
    float h = plane - ro.y*5.0;
    return h/rd.y;
}

float density(float3 pos,float l)
{    
    float den = 3.0 * FBM(pos * 0.2,l) - 2.0 + (pos.y - 3.0);
    float edge = 1.0 - smoothstep(MIN_HEIGHT, MAX_HEIGHT, pos.y);
    edge *= edge;
    den *= edge;
    den = clamp(den, 0.0, 4.0);
    
    return den;
}

float4 CloudsRender(  float4 sum,  float dif,  float den,  float3 c,  float t , float l)
{  

	float3 C = lerp(0.1+FOG_COLOR.rgb * 1.5,float3(0.9,0.9,0.9),l*0.35);
	
    float4 CloudsColor = float4( lerp( 1.15*C, C*0.9, dif ), den );
  
    CloudsColor.a *= 0.8*l*0.35;

    CloudsColor.rgb *= CloudsColor.a;
	
    return sum + CloudsColor*(1.0-sum.a);
	
}

float3 SkyRender(float2 uv,float l)
{
    float Slerp = length(uv/1.75);
	
	 float LightPos = l*0.35;
	 
    float3 Sky = lerp(float3(0.4,0.4,0.4),float3(0.2,0.5,1.0),LightPos);
   
    float3 To = lerp(0.21+FOG_COLOR.rgb * 1.5,Sky,LightPos);
	
    return lerp(Sky,To,Slerp / 1.3);
    
}



float3 Clouds(float3 ro, float3 rd, float t, float3 backCol,float l)
{   

	
	float4 sum = 0.0;
	
    float3 pos = ro + rd * t;
	
    for (int i = 0; i < 25; i++) {

        
        float den = density(pos,l);
        
 
            
             float dif =  clamp((den - density(pos - (l*0.35)-0.5,l))/0.6, 0.3, 1.0 );
            sum = CloudsRender( sum, dif, den, backCol, t ,l); 
        sum.a *= length(rd);
        
        t += max(0.01, 0.02 * t);
        pos = ro + rd * t;
    }
    
    sum = clamp(sum, 0.0, 1.0);
    
    float h = rd.y;
	
    sum.rgb = lerp(sum.rgb, backCol, exp(-100.*h*h) );
    
    return lerp(backCol, sum.xyz, sum.a);
}


float tri( float x){return clamp(abs(frac(x)-.5),0.001,0.52);}

float hash21( float2 n){ return frac(sin(dot(n, float2(12.9898, 4.1414))) * 43758.5453); }

float3 hash33(float3 p)
{
    p = frac(p * float3(443.8975,397.2973, 491.1871));
    p += dot(p.zxy, p.yxz+19.27);
    return frac(float3(p.x * p.y, p.z*p.x, p.y*p.z));
}

float triNoise2d( float2 p, float spd)
{
    float z=1.8;
    float z2=2.5;
	float rz = 0.;
    p *= 0.55*(p.x*0.2);
    float2 bp = p;
	for (float i=0.; i<10.; i++ )
	{
        float2 dg = (bp*1.85)*.75;
        dg *= 6.0;
        p -= dg/z2;

        bp *= 1.3;
        z2 *= .45;
        z *= .42;
		p *= 1.21 + (rz-1.0)*.02;
        
        rz += tri(p.x+tri(p.y))*z;
        p*= -1.0;
	}
    return clamp(1./pow(rz*29., 1.3),0.,.55);
}

float4 aurora(float3 ro, float3 rd)
{
    float4 col = 0.0;
    float4 avgCol = 0.0;
    
    for(float i=0.;i<32.;i++)
    {
        float of = 0.006*hash21(ro.xy)*smoothstep(0.,15., i);
        float pt = ((.8+pow(i,1.4)*.002)-ro.y)/(rd.y*2.+0.4);
        pt -= of;
    	float3 bpos = ro + pt*rd;
        float2 p = bpos.zx;
        float rzt = triNoise2d(p, 0.06);
        float4 col2 = float4(0,0,0, rzt);
        col2.rgb = (sin(1.-float3(2.15,-.5, 1.2)+i*0.043)*0.5+0.5)*rzt;
        avgCol =  lerp(avgCol, col2, .5);
        col += avgCol*exp2(-i*0.065 - 2.5)*smoothstep(0.,5., i);
        
    }
    
    col *= (clamp(rd.y*15.+.4,0.,1.));
    
    

    return col*1.5;

}

float3 bg( float3 rd)
{
    float sd = dot(normalize(float3(-0.5, -0.6, 0.9)), rd)*0.5+0.5;
    sd = pow(sd, 5.);
    float3 col = lerp(float3(0.05,0.1,0.2), float3(0.1,0.05,0.2), sd);
    return col*.63;
}

void main( in PS_Input PSInput, out PS_Output PSOutput )
{	
   
	float l = FOG_COLOR.r + FOG_COLOR.g + FOG_COLOR.b;
	
	float nr = pow(FOG_CONTROL.y,11.0);
   
    float2 p = PSInput.cloud.xz*4.0;

   
    float4 sky = float4(SkyRender(PSInput.cloud.xz*5.0,l),1.0);
	

//========================STARS=====================
    float t = sin(TIME * 2.0 * rand(-p/15.0)) * 0.5 + 0.5;
    
    float4 stars  = lerp(smoothstep(0.989, 1.0, rand(p/15.0)) * t,0.0,l*0.35);
   
   
    sky.rgb += stars.rgb*nr;
	 
//========================NORTHERN=====================
	float3 Northern = 0.0;

    float fade = smoothstep(0.,0.01,abs(normalize(float3(p.x,1.0,p.y)).y))*0.1+0.9;
    
    Northern = bg(normalize(float3(p.x,1.0,p.y)))*fade;
    

     float4 aur = smoothstep(0.,1.5,aurora(float3(0.0,0.0,-6.7),normalize(float3(p.x,1.0,p.y))))*fade;

     Northern = Northern*(1.-aur.a) + aur.rgb;
		

   sky.rgb += Northern*(1.0-l*0.35)*nr; 
	 
	 
//========================CLOUIDS=====================
    float3 ro = float3(0.0, 0.0, -1.0);
    

    float3 target = float3(1., 1.0, ro.z);
    
   
    target =  (target - ro) + ro;
    

    float3 rd =  normalize(float3(p.x, 0.75,p.y));
    
    float dist = planeIntersect(ro, rd, 3.3);	
    

    float3 col = Clouds(ro, rd, dist, sky.rgb,l);
	
//======================== SKY_PLANE =====================
	PSOutput.color.rgb = lerp( PSInput.color.rgb , col.rgb , (1.0 - PSInput.skya ) );

}