#include "ShaderConstants.fxh"
#include "util.fxh"


struct PS_Input
{
	float4 position : SV_Position;
	float3 pos : POS;
	float3 wpos : W_POS ;
	float fp : F_P;

#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
#endif

#ifdef BLEND
	float cameraDist : TEXCOORD_2;
#endif

#ifdef FOG
	float4 fogColor : FOG_COLOR;
#endif
};

struct PS_Output
{
	float4 color : SV_Target;
};

float d(float3 n,float3 l,float p) {
    return pow(dot(n,l) * 0.4 + 0.6,p);
}


float fc(float x) {

return ((x * (0.22 * x + 0.15 * 1.2 * 0.5) + 0.4 * 0.9 * 0.01 * 1.97) / (x * (0.22 * x + 0.5) + 0.4 * 0.9 * 0.2)) - 0.01 * 1.97 / 0.2;

}

float3 tone(float3 clr) {

float Luma = dot(clr, float3(0.5, 0.1, 0.1));
float3 Chroma = clr - Luma;
clr = (Chroma * 1.1) + Luma;

clr = float3(fc(clr.r), fc(clr.g), fc(clr.b)) / fc(1.2 / 1.3);
//clr += clr*0.1;
return clr;

}

ROOT_SIGNATURE
void main(in PS_Input PSInput, out PS_Output PSOutput)
{
float l = FOG_COLOR.r + FOG_COLOR.g + FOG_COLOR.b;
#ifdef BYPASS_PIXEL_SHADER
    PSOutput.color = float4(0.0f, 0.0f, 0.0f, 0.0f);
    return;
#else

#if USE_TEXEL_AA
	float4 diffuse = texture2D_AA(TEXTURE_0, TextureSampler0, PSInput.uv0 );
#else
	float4 diffuse = TEXTURE_0.Sample(TextureSampler0, PSInput.uv0);
#endif

#ifdef SEASONS_FAR
	diffuse.a = 1.0f;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
	#define ALPHA_THRESHOLD 0.05
	#else
	#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif



#ifndef SEASONS

#if !USE_ALPHA_TEST && !defined(BLEND)
	diffuse.a = PSInput.color.a;
#elif defined(BLEND)


#endif	
	diffuse.rgb *= PSInput.color.rgb;
#else
	float2 uv2 = PSInput.color.xy;
	diffuse.rgb *= lerp(1.0f, TEXTURE_2.Sample(TextureSampler2, uv2).rgb*2.0f, PSInput.color.b);
	diffuse.rgb *= PSInput.color.aaa;
	diffuse.a = 1.0f;
#endif

#ifdef BLEND	
diffuse *= 3.0;
diffuse.a = 0.6;
#ifdef NEAR_WATER
diffuse = float4(0.2,0.4,0.8,0.75);
#endif
#endif

#if !defined(ALWAYS_LIT)
	diffuse = diffuse * TEXTURE_1.Sample(TextureSampler1, PSInput.uv1);
#endif
float3 LightPos = float3(1.0*-l*0.35,0.0,0.0);
float3 N = normalize(cross(ddx(PSInput.pos),ddy(PSInput.pos))).xyz;


diffuse.rgb *= lerp(0.55+FOG_COLOR.rgb,1.0,l*0.35);
diffuse.rgb = tone(diffuse.rgb);
diffuse.rgb = pow(diffuse.rgb,0.6);


diffuse *= d(N,LightPos,0.5);

diffuse += float4(4.5,0.25,-5.0,1.0) * pow(PSInput.uv1.x*0.7,5.5);
//diffuse.rgb = lerp(diffuse.rgb,FOG_COLOR,clamp(PSInput.fp/10.0,0.0,0.75));
#ifdef FOG
	diffuse.rgb = lerp( diffuse.rgb, PSInput.fogColor.rgb, PSInput.fogColor.a );
#endif

	PSOutput.color = diffuse;

#ifdef VR_MODE
	// On Rift, the transition from 0 brightness to the lowest 8 bit value is abrupt, so clamp to 
	// the lowest 8 bit value.
	PSOutput.color = max(PSOutput.color, 1 / 255.0f);
#endif

#endif // BYPASS_PIXEL_SHADER
}

